package info.frostfs.sdk.examples;

import info.frostfs.sdk.ArrayHelper;
import info.frostfs.sdk.FrostFSClient;
import info.frostfs.sdk.dto.ape.*;
import info.frostfs.sdk.dto.chain.ChainTarget;
import info.frostfs.sdk.dto.container.Container;
import info.frostfs.sdk.dto.container.ContainerId;
import info.frostfs.sdk.dto.netmap.PlacementPolicy;
import info.frostfs.sdk.dto.netmap.Replica;
import info.frostfs.sdk.dto.object.ObjectAttribute;
import info.frostfs.sdk.dto.object.ObjectFrostFS;
import info.frostfs.sdk.dto.object.ObjectHeader;
import info.frostfs.sdk.dto.object.ObjectId;
import info.frostfs.sdk.enums.ObjectType;
import info.frostfs.sdk.enums.RuleMatchType;
import info.frostfs.sdk.enums.RuleStatus;
import info.frostfs.sdk.enums.TargetType;
import info.frostfs.sdk.jdo.parameters.CallContext;
import info.frostfs.sdk.jdo.parameters.ape.PrmApeChainAdd;
import info.frostfs.sdk.jdo.parameters.container.PrmContainerCreate;
import info.frostfs.sdk.jdo.parameters.container.PrmContainerDelete;
import info.frostfs.sdk.jdo.parameters.object.PrmObjectClientCutPut;
import info.frostfs.sdk.jdo.parameters.object.PrmObjectDelete;
import info.frostfs.sdk.jdo.parameters.object.PrmObjectGet;
import info.frostfs.sdk.jdo.parameters.object.PrmObjectSearch;
import info.frostfs.sdk.utils.WaitUtil;
import org.apache.commons.lang3.ArrayUtils;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

import static java.util.Objects.isNull;

public class ObjectExample {

    public static void testObject(FrostFSClient frostFSClient, CallContext callContext) {
        var placementPolicy = new PlacementPolicy(new Replica[]{new Replica(3)}, true, 1);
        var prmContainerCreate = new PrmContainerCreate(new Container(placementPolicy));
        prmContainerCreate.getContainer().getAttributes().put("с", UUID.randomUUID().toString());

        var containerId = frostFSClient.createContainer(prmContainerCreate, callContext);

        frostFSClient.addChain(generatePrmApeChainAdd(containerId), callContext);

        //Some wait
        WaitUtil.sleep(8000);

        //CREATE
        ObjectId oid;
        try (FileInputStream file = new FileInputStream("/home/ori/Desktop/DemoExample/cat.jpg")) {
            var attribute = new ObjectAttribute("Filename", "cat.jpg");
            var cat = new ObjectHeader(containerId, ObjectType.REGULAR, attribute);
            var prmObjectPut = PrmObjectClientCutPut.builder()
                    .objectHeader(cat)
                    .payload(file)
                    .bufferMaxSize(1024)
                    .build();
            oid = frostFSClient.putClientCutObject(prmObjectPut, new CallContext(10, TimeUnit.MINUTES));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        //SEARCH
        var prmObjectSearch = new PrmObjectSearch(containerId);
        var objects = frostFSClient.searchObjects(prmObjectSearch, callContext);

        //GET
        var prmObjectGet = new PrmObjectGet(containerId, oid);
        ObjectFrostFS object = frostFSClient.getObject(prmObjectGet, callContext);

        var reader = object.getObjectReader();
        var chunk = reader.readChunk();
        var length = chunk.length;
        byte[] buffer = null;
        while (length > 0) {
            buffer = isNull(buffer) ? chunk : ArrayHelper.concat(buffer, chunk);

            chunk = object.getObjectReader().readChunk();
            length = ArrayUtils.isEmpty(chunk) ? 0 : chunk.length;
        }

        try (FileOutputStream fos = new FileOutputStream("/home/ori/Desktop/DemoExample/newCat.jpg")) {
            fos.write(buffer);
        } catch (Exception ignored) {
        }

        //DELETE
        var prmObjectDelete = new PrmObjectDelete(containerId, oid);
        frostFSClient.deleteObject(prmObjectDelete, callContext);

        var prmContainerDelete = new PrmContainerDelete(containerId);
        frostFSClient.deleteContainer(prmContainerDelete, callContext);
    }

    private static PrmApeChainAdd generatePrmApeChainAdd(ContainerId containerId) {
        var chainTarget = new ChainTarget(containerId.getValue(), TargetType.CONTAINER);
        var resources = new Resources(false, new String[]{"native:object/*"});
        var actions = new Actions(false, new String[]{"*"});
        var rule = new Rule();
        rule.setStatus(RuleStatus.ALLOW);
        rule.setResources(resources);
        rule.setActions(actions);
        rule.setAny(false);
        rule.setConditions(new Condition[]{});


        var chain = new Chain();
        chain.setId(ArrayUtils.toObject("chain-id-test".getBytes(StandardCharsets.UTF_8)));
        chain.setRules(new Rule[]{rule});
        chain.setMatchType(RuleMatchType.DENY_PRIORITY);

        return new PrmApeChainAdd(chain, chainTarget);
    }
}
